# SPDX-FileCopyrightText: Peter Pentchev <roam@ringlet.net>
# SPDX-License-Identifier: BSD-2-Clause
"""Common definitions for the turtle-dragon library."""

from __future__ import annotations

import dataclasses
import enum
import typing


if typing.TYPE_CHECKING:
    import logging
    from typing import Final, Self


VERSION: Final = "0.1.0"
"""The turtle-dragon library version, semver-like."""


FEATURES: Final = {
    "turtle-dragon": VERSION,
}
"""The list of features supported by the turtle-dragon library."""


class Turn(enum.StrEnum):
    """The direction of the turn that we must make now."""

    LEFT = "left"
    """Turn left."""

    RIGHT = "right"
    """Turn right."""

    def converse(self) -> Self:
        """Turn the other way."""
        # https://github.com/python/mypy/issues/18029
        return Turn.LEFT if self == Turn.RIGHT else Turn.RIGHT  # type: ignore[return-value]


@dataclasses.dataclass(frozen=True)
class Config:
    """Runtime configuration for the turtle-dragon library."""

    cache: dict[int, Turn]
    """The cached turns for relatively small values of the generation."""

    log: logging.Logger
    """The logger to send diagnostic, informational, and error messages to."""
