# SPDX-FileCopyrightText: Peter Pentchev <roam@ringlet.net>
# SPDX-License-Identifier: BSD-2-Clause
"""Test calculating the turn directions."""

from __future__ import annotations

import typing

import pytest

from turtle_dragon import calc
from turtle_dragon import defs
from turtle_dragon import util


if typing.TYPE_CHECKING:
    from typing import Final


@pytest.mark.parametrize(
    ("current", "expected"),
    [
        (defs.Turn.LEFT, defs.Turn.RIGHT),
        (defs.Turn.RIGHT, defs.Turn.LEFT),
    ],
)
def test_turn_converse(*, current: defs.Turn, expected: defs.Turn) -> None:
    """Make sure we know how to turn the other way."""
    assert current != expected
    assert current.converse() == expected


@pytest.mark.parametrize(
    ("generation", "turn", "cached"),
    [
        (1, defs.Turn.LEFT, {1}),
        (2, defs.Turn.LEFT, {2}),
        (3, defs.Turn.RIGHT, {1, 3}),
        (4, defs.Turn.LEFT, {4}),
        (5, defs.Turn.LEFT, {1, 3, 5}),
        (6, defs.Turn.RIGHT, {2, 6}),
        (7, defs.Turn.RIGHT, {1, 7}),
        (8, defs.Turn.LEFT, {8}),
        (9, defs.Turn.LEFT, {1, 7, 9}),
        (10, defs.Turn.LEFT, {2, 6, 10}),
        (11, defs.Turn.RIGHT, {1, 3, 5, 11}),
        (12, defs.Turn.RIGHT, {4, 12}),
        (13, defs.Turn.LEFT, {1, 3, 13}),
        (14, defs.Turn.RIGHT, {2, 14}),
        (15, defs.Turn.RIGHT, {1, 15}),
        (16, defs.Turn.LEFT, {16}),
        (42, defs.Turn.LEFT, {2, 6, 10, 22, 42}),
        (616, defs.Turn.LEFT, {8, 24, 104, 408, 616}),
        (64, defs.Turn.LEFT, {64}),
        (256, defs.Turn.LEFT, {256}),
        (2025, defs.Turn.LEFT, {1, 7, 9, 23, 2025}),
        (131072, defs.Turn.LEFT, set()),
        (1048576, defs.Turn.LEFT, set()),
        (20250211, defs.Turn.RIGHT, set({1, 3, 29, 99, 413, 65123})),
    ],
)
def test_turn(*, generation: int, turn: defs.Turn, cached: set[int]) -> None:
    """Make sure we calculate the correct turn direction."""
    cfg: Final = defs.Config(
        cache={},
        log=util.build_logger(name="test-turn", quiet=False, verbose=True),
    )
    assert calc.turn(cfg, generation) == turn
    assert set(cfg.cache) == cached
